#This makes 3 states and one neuron, and uses two spike sources to move
#through.  First it turns on the first state, and fires the one neuron.
#Then it half turns on the second state and third state.  The second
#state comes on (turning off the first), but not the third state.

from nealCoverClass import NealCoverFunctions
import pyNN.nest as sim

import cPickle as pickle
import numpy as np

from stateMachineClass import FSAHelperFunctions

#---Below here are functions to test the system---
RUN_DURATION = 300
#initialize the simulator. 
def init():
    sim.setup(timestep=neal.DELAY,min_delay=neal.DELAY,
                    max_delay=neal.DELAY, debug=0)

def printPklSpikes(fileName):
    fileHandle = open(fileName)
    neoObj = pickle.load(fileHandle)
    segments = neoObj.segments
    segment = segments[0]
    spikeTrains = segment.spiketrains
    neurons = len(spikeTrains)
    for neuronNum in range (0,neurons):
        if (len(spikeTrains[neuronNum])>0):
            spikes = spikeTrains[neuronNum]
            for spike in range (0,len(spikes)):
                print neuronNum, spikes[spike]
    fileHandle.close()
    

def createFourInputs():
    inputSpikeTimes0 = [10.0]
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')

    inputSpikeTimes1 = [100.0]
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeGen1=sim.Population(1,sim.SpikeSourceArray,spikeArray1,
                                   label='inputSpikes_1')

    inputSpikeTimes2 = [150.0]
    spikeArray2 = {'spike_times': [inputSpikeTimes2]}
    spikeGen2=sim.Population(1,sim.SpikeSourceArray,spikeArray2,
                                   label='inputSpikes_2')

    inputSpikeTimes3 = [200.0]
    spikeArray3 = {'spike_times': [inputSpikeTimes3]}
    spikeGen3=sim.Population(1,sim.SpikeSourceArray,spikeArray3,
                                   label='inputSpikes_3')

    return [spikeGen0,spikeGen1,spikeGen2,spikeGen3]


def createStateCells():
    numNeurons = fsa.CA_SIZE * 4
    cells=sim.Population(numNeurons,sim.IF_cond_exp,fsa.CELL_PARAMS)
    return cells

def createNeurons(numNeurons):
    cells=sim.Population(numNeurons,sim.IF_cond_exp,fsa.CELL_PARAMS)
    return cells

def setupRecording(cells):
    cells.record({'spikes','v'})

def printResults(stateCells,neuronCells):
    if  (neal.simulator == 'nest'):
        stateCells.printSpikes('state.pkl')
        neuronCells.printSpikes('neurons.pkl')

##--main---
simName = "nest"
spinnVersion = -1
neal = NealCoverFunctions(simName,sim,spinnVersion)
fsa = FSAHelperFunctions(simName,sim,neal,spinnVersion)
init()
spikeGenerators =  createFourInputs()

stateCells = createStateCells()
setupRecording(stateCells)

neuronCells = createNeurons(2)
setupRecording(neuronCells)

#make a state
fsa.makeCA(stateCells,0)
fsa.makeCA(stateCells,1)
fsa.makeCA(stateCells,2)

#s0 + second spikeSource0 move to s1
fsa.stateHalfTurnsOnState(stateCells,0,stateCells,1)
fsa.stateTurnsOffState(stateCells,1,stateCells,0)

#play with the state using spike sources
fsa.turnOnStateFromSpikeSource(spikeGenerators[0],stateCells,0) #turn on s0
fsa.halfTurnOnStateFromSpikeSource(spikeGenerators[1],stateCells,1)
fsa.halfTurnOnStateFromSpikeSource(spikeGenerators[1],stateCells,2)
fsa.turnOnOneNeuronFromSpikeSource(spikeGenerators[0],neuronCells,1) #turn on n1

neal.nealApplyProjections()
sim.run(RUN_DURATION)

printResults(stateCells,neuronCells)
